# frozen_string_literal: true

# Handles modeling of Address when performing requests (create / update)
class AvataxClient::Request::Address < AvataxClient::Request::Base
  property :city, required: :not_geocoded?
  property :country, required: :not_geocoded?
  # Values: ShipFrom, ShipTo, PointOfOrderAcceptance, PointOfOrderOrigin, SingleLocation
  property :label, required: true
  property :latitude
  property :longitude
  property :line1, required: :not_geocoded?
  property :line2
  property :line3
  property :region, required: :not_geocoded?
  property :postal_code, required: :not_geocoded?, from: :postalCode

  ## Class Methods
  class << self
    # Transform a colleciton of {Address} into a format compatible with Avatax.
    # @param [Array<Address>]
    # @return [Hash]
    def collection_to_body(addresses = [])
      addresses.reduce({}) { |a, e| a.merge!(e.to_body) }
    end
  end

  ## Instance Methods

  # Transform Hash into a version compatible with Avatax.
  # Removes `label` attribute from hash, as it is used as the parent key in `addresses: {}`.
  # @return [Hash]
  def to_body
    hash = super

    label = hash[:label]
    hash.delete(:label)
    hash = { label => hash }

    hash
  end

  private

  def not_geocoded?
    return true if latitude.nil? || longitude.nil?
    false
  end
end
