# frozen_string_literal: true

# Represents a line item on a {Response::Transaction}.
# Responsible for reporting dollar amounts, quantity and tax code.
class AvataxClient::Response::LineItem < AvataxClient::Response::Base
  property :amount, coerce: Float, from: :lineAmount, required: true
  property :details, coerce: Array[AvataxClient::Response::LineItemDetail], required: true
  property :item_code, from: :itemCode
  property :number, coerce: Integer, from: :lineNumber, required: true
  property :quantity, required: true
  property :tax, coerce: Float, required: true
  property :tax_calculated, coerce: Float, from: :taxCalculated, required: true
  property :tax_code, from: :taxCode, required: true
  property :taxable_amount, coerce: Float, from: :taxableAmount, required: true

  # Effective tax rate is determined by summing the rate from the details.
  # @return [float]
  def rate
    @rate ||= details.map(&:rate).reduce(&:+)
  end
end
