# frozen_string_literal: true

# Transform a hash into a format compatible with Avatax.
module AvataxClient::ReverseCoercion
  def self.included(base)
    base.extend(ClassMethods)
  end

  ## Class methods
  module ClassMethods
    # Override to allow for Modification at the colleciton level.
    # Defaults to calling `.to_body` for each element.
    # @param values [Array] collection items to be transformed.
    # @return [Array]
    def collection_to_body(values)
      values.map(&:to_body)
    end
  end

  # Build a hash that can be used in the request body to Avatax API
  # @return [Hash]
  def to_body
    hash = to_hash
    hash = translate_names(hash)
    hash = transform_collections(hash)
    hash
  end

  private

  def transform_collections(hash)
    self.class.key_coercions.each do |k, v|
      hash[k] = if v.is_a?(Array)
                  v.first.collection_to_body(hash[k])
                elsif v.respond_to?(:to_body)
                  v.to_body
                else
                  hash[k]
                end
    end
    hash
  end

  def translate_names(hash)
    self.class.inverse_translations.each do |k, v|
      hash[v] = hash[k]
      hash.delete(k)
    end
    hash
  end
end
